/*
 * Decompiled with CFR 0.0.
 * 
 * Could not load the following classes:
 *  java.io.IOException
 *  java.lang.Integer
 *  java.lang.Math
 *  java.lang.Object
 *  java.text.DateFormat
 *  java.text.NumberFormat
 *  java.util.ArrayList
 *  java.util.HashMap
 *  java.util.Iterator
 *  jxl.biff.BuiltInStyle
 *  jxl.biff.FormatRecord
 *  jxl.biff.PaletteRecord
 *  jxl.biff.XFRecord
 */
package jxl.biff;

import java.io.IOException;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import jxl.biff.BuiltInStyle;
import jxl.biff.ByteData;
import jxl.biff.DisplayFormat;
import jxl.biff.Fonts;
import jxl.biff.FormatRecord;
import jxl.biff.IndexMapping;
import jxl.biff.NumFormatRecordsException;
import jxl.biff.PaletteRecord;
import jxl.biff.XFRecord;
import jxl.common.Assert;
import jxl.common.Logger;
import jxl.format.Colour;
import jxl.format.RGB;
import jxl.write.biff.File;

public class FormattingRecords {
    private static final int customFormatStartIndex = 164;
    private static Logger logger = Logger.getLogger(FormattingRecords.class);
    private static final int maxFormatRecordsIndex = 441;
    private static final int minXFRecords = 21;
    private Fonts fonts;
    private HashMap formats = new HashMap(10);
    private ArrayList formatsList = new ArrayList(10);
    private int nextCustomIndexNumber;
    private PaletteRecord palette;
    private ArrayList xfRecords = new ArrayList(10);

    public FormattingRecords(Fonts fonts) {
        this.fonts = fonts;
        this.nextCustomIndexNumber = 164;
    }

    public final void addFormat(DisplayFormat displayFormat) throws NumFormatRecordsException {
        if (displayFormat.isInitialized() && displayFormat.getFormatIndex() >= 441) {
            logger.warn("Format index exceeds Excel maximum - assigning custom number");
            displayFormat.initialize(this.nextCustomIndexNumber);
            this.nextCustomIndexNumber = 1 + this.nextCustomIndexNumber;
        }
        if (!displayFormat.isInitialized()) {
            displayFormat.initialize(this.nextCustomIndexNumber);
            this.nextCustomIndexNumber = 1 + this.nextCustomIndexNumber;
        }
        if (this.nextCustomIndexNumber <= 441) {
            if (displayFormat.getFormatIndex() >= this.nextCustomIndexNumber) {
                this.nextCustomIndexNumber = 1 + displayFormat.getFormatIndex();
            }
            if (!displayFormat.isBuiltIn()) {
                this.formatsList.add((Object)displayFormat);
                this.formats.put((Object)new Integer(displayFormat.getFormatIndex()), (Object)displayFormat);
            }
            return;
        }
        this.nextCustomIndexNumber = 441;
        throw new NumFormatRecordsException();
    }

    public final void addStyle(XFRecord xFRecord) throws NumFormatRecordsException {
        if (!xFRecord.isInitialized()) {
            xFRecord.initialize(this.xfRecords.size(), this, this.fonts);
            this.xfRecords.add((Object)xFRecord);
            return;
        }
        if (xFRecord.getXFIndex() >= this.xfRecords.size()) {
            this.xfRecords.add((Object)xFRecord);
        }
    }

    public RGB getColourRGB(Colour colour) {
        PaletteRecord paletteRecord = this.palette;
        if (paletteRecord == null) {
            return colour.getDefaultRGB();
        }
        return paletteRecord.getColourRGB(colour);
    }

    public final DateFormat getDateFormat(int n) {
        XFRecord xFRecord = (XFRecord)this.xfRecords.get(n);
        if (xFRecord.isDate()) {
            return xFRecord.getDateFormat();
        }
        FormatRecord formatRecord = (FormatRecord)this.formats.get((Object)new Integer(xFRecord.getFormatRecord()));
        if (formatRecord == null) {
            return null;
        }
        boolean bl = formatRecord.isDate();
        DateFormat dateFormat = null;
        if (bl) {
            dateFormat = formatRecord.getDateFormat();
        }
        return dateFormat;
    }

    protected final Fonts getFonts() {
        return this.fonts;
    }

    FormatRecord getFormatRecord(int n) {
        return (FormatRecord)this.formats.get((Object)new Integer(n));
    }

    public final NumberFormat getNumberFormat(int n) {
        XFRecord xFRecord = (XFRecord)this.xfRecords.get(n);
        if (xFRecord.isNumber()) {
            return xFRecord.getNumberFormat();
        }
        FormatRecord formatRecord = (FormatRecord)this.formats.get((Object)new Integer(xFRecord.getFormatRecord()));
        if (formatRecord == null) {
            return null;
        }
        boolean bl = formatRecord.isNumber();
        NumberFormat numberFormat = null;
        if (bl) {
            numberFormat = formatRecord.getNumberFormat();
        }
        return numberFormat;
    }

    protected final int getNumberOfFormatRecords() {
        return this.formatsList.size();
    }

    public PaletteRecord getPalette() {
        return this.palette;
    }

    public final XFRecord getXFRecord(int n) {
        return (XFRecord)this.xfRecords.get(n);
    }

    public final boolean isDate(int n) {
        XFRecord xFRecord = (XFRecord)this.xfRecords.get(n);
        if (xFRecord.isDate()) {
            return true;
        }
        FormatRecord formatRecord = (FormatRecord)this.formats.get((Object)new Integer(xFRecord.getFormatRecord()));
        if (formatRecord == null) {
            return false;
        }
        return formatRecord.isDate();
    }

    public IndexMapping rationalize(IndexMapping indexMapping, IndexMapping indexMapping2) {
        for (XFRecord xFRecord : this.xfRecords) {
            if (xFRecord.getFormatRecord() >= 164) {
                xFRecord.setFormatIndex(indexMapping2.getNewIndex(xFRecord.getFormatRecord()));
            }
            xFRecord.setFontIndex(indexMapping.getNewIndex(xFRecord.getFontIndex()));
        }
        int n = 21;
        ArrayList arrayList = new ArrayList(n);
        IndexMapping indexMapping3 = new IndexMapping(this.xfRecords.size());
        int n2 = Math.min((int)n, (int)this.xfRecords.size());
        for (int i = 0; i < n2; ++i) {
            arrayList.add(this.xfRecords.get(i));
            indexMapping3.setMapping(i, i);
        }
        if (n2 < n) {
            logger.warn("There are less than the expected minimum number of XF records");
            return indexMapping3;
        }
        int n3 = 0;
        while (n < this.xfRecords.size()) {
            XFRecord xFRecord = (XFRecord)this.xfRecords.get(n);
            Iterator iterator = arrayList.iterator();
            int n4 = n3;
            boolean bl = false;
            while (iterator.hasNext() && !bl) {
                XFRecord xFRecord2 = (XFRecord)iterator.next();
                if (!xFRecord2.equals((Object)xFRecord)) continue;
                indexMapping3.setMapping(n, indexMapping3.getNewIndex(xFRecord2.getXFIndex()));
                ++n4;
                bl = true;
            }
            if (!bl) {
                arrayList.add((Object)xFRecord);
                indexMapping3.setMapping(n, n - n4);
            }
            ++n;
            n3 = n4;
        }
        Iterator iterator = this.xfRecords.iterator();
        while (iterator.hasNext()) {
            ((XFRecord)iterator.next()).rationalize(indexMapping3);
        }
        this.xfRecords = arrayList;
        return indexMapping3;
    }

    public IndexMapping rationalizeDisplayFormats() {
        ArrayList arrayList = new ArrayList();
        IndexMapping indexMapping = new IndexMapping(this.nextCustomIndexNumber);
        Iterator iterator = this.formatsList.iterator();
        int n = 0;
        while (iterator.hasNext()) {
            DisplayFormat displayFormat = (DisplayFormat)iterator.next();
            Assert.verify(true ^ displayFormat.isBuiltIn());
            Iterator iterator2 = arrayList.iterator();
            int n2 = n;
            boolean bl = false;
            while (iterator2.hasNext() && !bl) {
                DisplayFormat displayFormat2 = (DisplayFormat)iterator2.next();
                if (!displayFormat2.equals((Object)displayFormat)) continue;
                indexMapping.setMapping(displayFormat.getFormatIndex(), indexMapping.getNewIndex(displayFormat2.getFormatIndex()));
                ++n2;
                bl = true;
            }
            if (!bl) {
                arrayList.add((Object)displayFormat);
                if (displayFormat.getFormatIndex() - n2 > 441) {
                    logger.warn("Too many number formats - using default format.");
                }
                indexMapping.setMapping(displayFormat.getFormatIndex(), displayFormat.getFormatIndex() - n2);
            }
            n = n2;
        }
        this.formatsList = arrayList;
        for (DisplayFormat displayFormat : this.formatsList) {
            displayFormat.initialize(indexMapping.getNewIndex(displayFormat.getFormatIndex()));
        }
        return indexMapping;
    }

    public IndexMapping rationalizeFonts() {
        return this.fonts.rationalize();
    }

    public void setColourRGB(Colour colour, int n, int n2, int n3) {
        if (this.palette == null) {
            this.palette = new PaletteRecord();
        }
        this.palette.setColourRGB(colour, n, n2, n3);
    }

    public void setPalette(PaletteRecord paletteRecord) {
        this.palette = paletteRecord;
    }

    public void write(File file) throws IOException {
        Iterator iterator = this.formatsList.iterator();
        while (iterator.hasNext()) {
            file.write((ByteData)((FormatRecord)iterator.next()));
        }
        Iterator iterator2 = this.xfRecords.iterator();
        while (iterator2.hasNext()) {
            file.write((ByteData)((XFRecord)iterator2.next()));
        }
        file.write((ByteData)new BuiltInStyle(16, 3));
        file.write((ByteData)new BuiltInStyle(17, 6));
        file.write((ByteData)new BuiltInStyle(18, 4));
        file.write((ByteData)new BuiltInStyle(19, 7));
        file.write((ByteData)new BuiltInStyle(0, 0));
        file.write((ByteData)new BuiltInStyle(20, 5));
    }
}

