/*
 * Decompiled with CFR 0.0.
 * 
 * Could not load the following classes:
 *  java.io.File
 *  java.io.IOException
 *  java.lang.Integer
 *  java.lang.Math
 *  java.lang.Object
 *  java.lang.String
 *  java.lang.StringBuffer
 *  java.lang.StringBuilder
 *  java.lang.System
 *  java.net.URL
 *  java.util.ArrayList
 *  java.util.Comparator
 *  java.util.Iterator
 *  java.util.TreeSet
 *  java.util.regex.Pattern
 */
package jxl.write.biff;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.regex.Pattern;
import jxl.Cell;
import jxl.CellFeatures;
import jxl.CellReferenceHelper;
import jxl.CellType;
import jxl.CellView;
import jxl.HeaderFooter;
import jxl.Hyperlink;
import jxl.Image;
import jxl.LabelCell;
import jxl.Range;
import jxl.Sheet;
import jxl.SheetSettings;
import jxl.WorkbookSettings;
import jxl.biff.AutoFilter;
import jxl.biff.BaseCellFeatures;
import jxl.biff.CellFinder;
import jxl.biff.ConditionalFormat;
import jxl.biff.DVParser;
import jxl.biff.DataValidation;
import jxl.biff.EmptyCell;
import jxl.biff.FontRecord;
import jxl.biff.FormattingRecords;
import jxl.biff.IndexMapping;
import jxl.biff.NumFormatRecordsException;
import jxl.biff.SheetRangeImpl;
import jxl.biff.WorkspaceInformationRecord;
import jxl.biff.XFRecord;
import jxl.biff.drawing.Chart;
import jxl.biff.drawing.ComboBox;
import jxl.biff.drawing.Drawing;
import jxl.biff.drawing.DrawingGroup;
import jxl.biff.drawing.DrawingGroupObject;
import jxl.common.Assert;
import jxl.common.Logger;
import jxl.format.CellFormat;
import jxl.format.Font;
import jxl.format.PageOrientation;
import jxl.format.PaperSize;
import jxl.write.Blank;
import jxl.write.Label;
import jxl.write.WritableCell;
import jxl.write.WritableCellFeatures;
import jxl.write.WritableCellFormat;
import jxl.write.WritableFont;
import jxl.write.WritableHyperlink;
import jxl.write.WritableImage;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;
import jxl.write.biff.ButtonPropertySetRecord;
import jxl.write.biff.CellValue;
import jxl.write.biff.ColumnInfoRecord;
import jxl.write.biff.File;
import jxl.write.biff.FooterRecord;
import jxl.write.biff.HeaderRecord;
import jxl.write.biff.HyperlinkRecord;
import jxl.write.biff.JxlWriteException;
import jxl.write.biff.MergedCells;
import jxl.write.biff.PLSRecord;
import jxl.write.biff.RowRecord;
import jxl.write.biff.RowsExceededException;
import jxl.write.biff.SharedStrings;
import jxl.write.biff.SheetCopier;
import jxl.write.biff.SheetWriter;
import jxl.write.biff.Styles;
import jxl.write.biff.WritableSheetCopier;
import jxl.write.biff.WritableSheetImpl;
import jxl.write.biff.WritableWorkbookImpl;

class WritableSheetImpl
implements WritableSheet {
    private static final char[] illegalSheetNameCharacters;
    private static final String[] imageTypes;
    private static Logger logger;
    private static final int maxSheetNameLength = 31;
    private static final int numRowsPerSheet = 65536;
    private static final int rowGrowSize = 10;
    private AutoFilter autoFilter;
    private TreeSet autosizedColumns;
    private ButtonPropertySetRecord buttonPropertySet;
    private boolean chartOnly;
    private ArrayList columnBreaks;
    private TreeSet columnFormats;
    private ComboBox comboBox;
    private ArrayList conditionalFormats;
    private DataValidation dataValidation;
    private ArrayList drawings;
    private boolean drawingsModified;
    private FormattingRecords formatRecords;
    private ArrayList hyperlinks;
    private ArrayList images;
    private int maxColumnOutlineLevel;
    private int maxRowOutlineLevel;
    private MergedCells mergedCells;
    private String name;
    private int numColumns;
    private int numRows;
    private File outputFile;
    private PLSRecord plsRecord;
    private ArrayList rowBreaks;
    private RowRecord[] rows;
    private SheetSettings settings;
    private SharedStrings sharedStrings;
    private SheetWriter sheetWriter;
    private ArrayList validatedCells;
    private WritableWorkbookImpl workbook;
    private WorkbookSettings workbookSettings;

    static {
        logger = Logger.getLogger(WritableSheetImpl.class);
        illegalSheetNameCharacters = new char[]{'*', ':', '?', '\\'};
        imageTypes = new String[]{"png"};
    }

    public WritableSheetImpl(String string, File file, FormattingRecords formattingRecords, SharedStrings sharedStrings, WorkbookSettings workbookSettings, WritableWorkbookImpl writableWorkbookImpl) {
        this.name = this.validateName(string);
        this.outputFile = file;
        this.rows = new RowRecord[0];
        this.numRows = 0;
        this.numColumns = 0;
        this.chartOnly = false;
        this.workbook = writableWorkbookImpl;
        this.formatRecords = formattingRecords;
        this.sharedStrings = sharedStrings;
        this.workbookSettings = workbookSettings;
        this.drawingsModified = false;
        this.columnFormats = new TreeSet(new Comparator(){

            public int compare(Object object, Object object2) {
                if (object == object2) {
                    return 0;
                }
                Assert.verify(object instanceof ColumnInfoRecord);
                Assert.verify(object2 instanceof ColumnInfoRecord);
                ColumnInfoRecord columnInfoRecord = (ColumnInfoRecord)object;
                ColumnInfoRecord columnInfoRecord2 = (ColumnInfoRecord)object2;
                return columnInfoRecord.getColumn() - columnInfoRecord2.getColumn();
            }

            public boolean equals(Object object) {
                return object == this;
            }
        });
        this.autosizedColumns = new TreeSet();
        this.hyperlinks = new ArrayList();
        this.mergedCells = new MergedCells(this);
        this.rowBreaks = new ArrayList();
        this.columnBreaks = new ArrayList();
        this.drawings = new ArrayList();
        this.images = new ArrayList();
        this.conditionalFormats = new ArrayList();
        this.validatedCells = new ArrayList();
        this.settings = new SheetSettings(this);
        this.sheetWriter = new SheetWriter(this.outputFile, this, this.workbookSettings);
    }

    private void autosizeColumn(int n) {
        ColumnInfoRecord columnInfoRecord = this.getColumnInfo(n);
        Font font = columnInfoRecord.getCellFormat().getFont();
        Font font2 = WritableWorkbook.NORMAL_STYLE.getFont();
        int n2 = 0;
        for (int i = 0; i < this.numRows; ++i) {
            RowRecord[] arrrowRecord = this.rows;
            RowRecord rowRecord = arrrowRecord[i];
            Cell cell = null;
            if (rowRecord != null) {
                cell = arrrowRecord[i].getCell(n);
            }
            if (cell == null) continue;
            String string = cell.getContents();
            Font font3 = cell.getCellFormat().getFont();
            if (font3.equals((Object)font2)) {
                font3 = font;
            }
            int n3 = font3.getPointSize();
            int n4 = string.length();
            if (font3.isItalic() || font3.getBoldWeight() > 400) {
                n4 += 2;
            }
            n2 = Math.max((int)n2, (int)(256 * (n4 * n3)));
        }
        columnInfoRecord.setWidth(n2 / font2.getPointSize());
    }

    private void autosizeColumns() {
        Iterator iterator = this.autosizedColumns.iterator();
        while (iterator.hasNext()) {
            this.autosizeColumn((Integer)iterator.next());
        }
    }

    private DrawingGroupObject[] getDrawings() {
        Object[] arrobject = new DrawingGroupObject[this.drawings.size()];
        return (DrawingGroupObject[])this.drawings.toArray(arrobject);
    }

    private WorkspaceInformationRecord getWorkspaceOptions() {
        return this.sheetWriter.getWorkspaceOptions();
    }

    private String validateName(String string) {
        char[] arrc;
        if (string.length() > 31) {
            Logger logger = WritableSheetImpl.logger;
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("Sheet name ");
            stringBuilder.append(string);
            stringBuilder.append(" too long - truncating");
            logger.warn(stringBuilder.toString());
            string = string.substring(0, 31);
        }
        char c = string.charAt(0);
        int n = 0;
        if (c == '\'') {
            logger.warn("Sheet naming cannot start with ' - removing");
            string = string.substring(1);
        }
        while (n < (arrc = illegalSheetNameCharacters).length) {
            String string2 = string.replace(arrc[n], '@');
            if (string != string2) {
                Logger logger = WritableSheetImpl.logger;
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.append(illegalSheetNameCharacters[n]);
                stringBuilder.append(" is not a valid character within a sheet name - replacing");
                logger.warn(stringBuilder.toString());
            }
            ++n;
            string = string2;
        }
        return string;
    }

    @Override
    public void addCell(WritableCell writableCell) throws WriteException, RowsExceededException {
        if (writableCell.getType() == CellType.EMPTY && writableCell != null && writableCell.getCellFormat() == null) {
            return;
        }
        CellValue cellValue = (CellValue)writableCell;
        if (!cellValue.isReferenced()) {
            int n = writableCell.getRow();
            RowRecord rowRecord = this.getRowRecord(n);
            CellValue cellValue2 = rowRecord.getCell(cellValue.getColumn());
            boolean bl = cellValue2 != null && cellValue2.getCellFeatures() != null && cellValue2.getCellFeatures().getDVParser() != null && cellValue2.getCellFeatures().getDVParser().extendedCellsValidation();
            if (writableCell.getCellFeatures() != null && writableCell.getCellFeatures().hasDataValidation() && bl) {
                DVParser dVParser = cellValue2.getCellFeatures().getDVParser();
                Logger logger = WritableSheetImpl.logger;
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.append("Cannot add cell at ");
                stringBuilder.append(CellReferenceHelper.getCellReference(cellValue));
                stringBuilder.append(" because it is part of the shared cell validation group ");
                stringBuilder.append(CellReferenceHelper.getCellReference(dVParser.getFirstColumn(), dVParser.getFirstRow()));
                stringBuilder.append("-");
                stringBuilder.append(CellReferenceHelper.getCellReference(dVParser.getLastColumn(), dVParser.getLastRow()));
                logger.warn(stringBuilder.toString());
                return;
            }
            if (bl) {
                WritableCellFeatures writableCellFeatures = writableCell.getWritableCellFeatures();
                if (writableCellFeatures == null) {
                    writableCellFeatures = new WritableCellFeatures();
                    writableCell.setCellFeatures(writableCellFeatures);
                }
                writableCellFeatures.shareDataValidation(cellValue2.getCellFeatures());
            }
            rowRecord.addCell(cellValue);
            this.numRows = Math.max((int)(n + 1), (int)this.numRows);
            this.numColumns = Math.max((int)this.numColumns, (int)rowRecord.getMaxColumn());
            cellValue.setCellDetails(this.formatRecords, this.sharedStrings, this);
            return;
        }
        throw new JxlWriteException(JxlWriteException.cellReferenced);
    }

    @Override
    public void addColumnPageBreak(int n) {
        Iterator iterator = this.columnBreaks.iterator();
        boolean bl = false;
        while (iterator.hasNext() && !bl) {
            if ((Integer)iterator.next() != n) continue;
            bl = true;
        }
        if (!bl) {
            this.columnBreaks.add((Object)new Integer(n));
        }
    }

    void addDrawing(DrawingGroupObject drawingGroupObject) {
        this.drawings.add((Object)drawingGroupObject);
        Assert.verify(true ^ drawingGroupObject instanceof Drawing);
    }

    @Override
    public void addHyperlink(WritableHyperlink writableHyperlink) throws WriteException, RowsExceededException {
        String string;
        Cell cell = this.getCell(writableHyperlink.getColumn(), writableHyperlink.getRow());
        if (!writableHyperlink.isFile() && !writableHyperlink.isUNC()) {
            if (writableHyperlink.isURL()) {
                string = writableHyperlink.getContents();
                if (string == null) {
                    string = writableHyperlink.getURL().toString();
                }
            } else {
                string = writableHyperlink.isLocation() ? writableHyperlink.getContents() : null;
            }
        } else {
            string = writableHyperlink.getContents();
            if (string == null) {
                string = writableHyperlink.getFile().getPath();
            }
        }
        if (cell.getType() == CellType.LABEL) {
            Label label = (Label)cell;
            label.setString(string);
            WritableCellFormat writableCellFormat = new WritableCellFormat(label.getCellFormat());
            writableCellFormat.setFont(WritableWorkbook.HYPERLINK_FONT);
            label.setCellFormat(writableCellFormat);
        } else {
            this.addCell(new Label(writableHyperlink.getColumn(), writableHyperlink.getRow(), string, WritableWorkbook.HYPERLINK_STYLE));
        }
        for (int i = writableHyperlink.getRow(); i <= writableHyperlink.getLastRow(); ++i) {
            for (int j = writableHyperlink.getColumn(); j <= writableHyperlink.getLastColumn(); ++j) {
                RowRecord[] arrrowRecord;
                if (i == writableHyperlink.getRow() || j == writableHyperlink.getColumn() || this.rows.length >= writableHyperlink.getLastColumn() || (arrrowRecord = this.rows)[i] == null) continue;
                arrrowRecord[i].removeCell(j);
            }
        }
        writableHyperlink.initialize(this);
        this.hyperlinks.add((Object)writableHyperlink);
    }

    @Override
    public void addImage(WritableImage writableImage) {
        boolean bl;
        String string;
        java.io.File file = writableImage.getImageFile();
        int n = 1;
        if (file != null) {
            String[] arrstring;
            String string2 = file.getName();
            int n2 = string2.lastIndexOf(46);
            string = n2 != -1 ? string2.substring(n2 + n) : "";
            bl = false;
            for (int i = 0; i < (arrstring = imageTypes).length && !bl; ++i) {
                if (!string.equalsIgnoreCase(arrstring[i])) continue;
                bl = true;
            }
        } else {
            string = "?";
            bl = true;
        }
        if (bl) {
            this.workbook.addDrawing(writableImage);
            this.drawings.add((Object)writableImage);
            this.images.add((Object)writableImage);
            return;
        }
        StringBuffer stringBuffer = new StringBuffer("Image type ");
        stringBuffer.append(string);
        stringBuffer.append(" not supported.  Supported types are ");
        stringBuffer.append(imageTypes[0]);
        while (n < imageTypes.length) {
            stringBuffer.append(", ");
            stringBuffer.append(imageTypes[n]);
            ++n;
        }
        logger.warn(stringBuffer.toString());
    }

    @Override
    public void addRowPageBreak(int n) {
        Iterator iterator = this.rowBreaks.iterator();
        boolean bl = false;
        while (iterator.hasNext() && !bl) {
            if ((Integer)iterator.next() != n) continue;
            bl = true;
        }
        if (!bl) {
            this.rowBreaks.add((Object)new Integer(n));
        }
    }

    void addValidationCell(CellValue cellValue) {
        this.validatedCells.add((Object)cellValue);
    }

    @Override
    public void applySharedDataValidation(WritableCell writableCell, int n, int n2) throws WriteException {
        if (writableCell.getWritableCellFeatures() != null && writableCell.getWritableCellFeatures().hasDataValidation()) {
            int n3 = writableCell.getColumn();
            int n4 = writableCell.getRow();
            int n5 = -1 + this.numRows;
            int n6 = n4 + n2;
            int n7 = Math.min((int)n5, (int)n6);
            for (int i = n4; i <= n7; ++i) {
                RowRecord[] arrrowRecord = this.rows;
                if (arrrowRecord[i] == null) continue;
                int n8 = -1 + arrrowRecord[i].getMaxColumn();
                int n9 = n3 + n;
                int n10 = Math.min((int)n8, (int)n9);
                for (int j = n3; j <= n10; ++j) {
                    CellValue cellValue;
                    if (j == n3 && i == n4 || (cellValue = this.rows[i].getCell(j)) == null || cellValue.getWritableCellFeatures() == null || !cellValue.getWritableCellFeatures().hasDataValidation()) continue;
                    Logger logger = WritableSheetImpl.logger;
                    StringBuilder stringBuilder = new StringBuilder();
                    stringBuilder.append("Cannot apply data validation from ");
                    stringBuilder.append(CellReferenceHelper.getCellReference(n3, n4));
                    stringBuilder.append(" to ");
                    stringBuilder.append(CellReferenceHelper.getCellReference(n9, n6));
                    stringBuilder.append(" as cell ");
                    stringBuilder.append(CellReferenceHelper.getCellReference(j, i));
                    stringBuilder.append(" already has a data validation");
                    logger.warn(stringBuilder.toString());
                    return;
                }
            }
            WritableCellFeatures writableCellFeatures = writableCell.getWritableCellFeatures();
            writableCellFeatures.getDVParser().extendCellValidation(n, n2);
            for (int i = n4; i <= n6; ++i) {
                RowRecord rowRecord = this.getRowRecord(i);
                for (int j = n3; j <= n3 + n; ++j) {
                    if (j == n3 && i == n4) continue;
                    CellValue cellValue = rowRecord.getCell(j);
                    if (cellValue == null) {
                        Blank blank = new Blank(j, i);
                        WritableCellFeatures writableCellFeatures2 = new WritableCellFeatures();
                        writableCellFeatures2.shareDataValidation(writableCellFeatures);
                        blank.setCellFeatures(writableCellFeatures2);
                        this.addCell(blank);
                        continue;
                    }
                    WritableCellFeatures writableCellFeatures3 = cellValue.getWritableCellFeatures();
                    if (writableCellFeatures3 != null) {
                        writableCellFeatures3.shareDataValidation(writableCellFeatures);
                        continue;
                    }
                    WritableCellFeatures writableCellFeatures4 = new WritableCellFeatures();
                    writableCellFeatures4.shareDataValidation(writableCellFeatures);
                    cellValue.setCellFeatures(writableCellFeatures4);
                }
            }
            return;
        }
        Logger logger = WritableSheetImpl.logger;
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("Cannot extend data validation for ");
        stringBuilder.append(CellReferenceHelper.getCellReference(writableCell.getColumn(), writableCell.getRow()));
        stringBuilder.append(" as it has no data validation");
        logger.warn(stringBuilder.toString());
    }

    void checkMergedBorders() {
        this.sheetWriter.setWriteData(this.rows, this.rowBreaks, this.columnBreaks, this.hyperlinks, this.mergedCells, this.columnFormats, this.maxRowOutlineLevel, this.maxColumnOutlineLevel);
        this.sheetWriter.setDimensions(this.getRows(), this.getColumns());
        this.sheetWriter.checkMergedBorders();
    }

    void copy(Sheet sheet) {
        this.settings = new SheetSettings(sheet.getSettings(), this);
        SheetCopier sheetCopier = new SheetCopier(sheet, this);
        sheetCopier.setColumnFormats(this.columnFormats);
        sheetCopier.setFormatRecords(this.formatRecords);
        sheetCopier.setHyperlinks(this.hyperlinks);
        sheetCopier.setMergedCells(this.mergedCells);
        sheetCopier.setRowBreaks(this.rowBreaks);
        sheetCopier.setColumnBreaks(this.columnBreaks);
        sheetCopier.setSheetWriter(this.sheetWriter);
        sheetCopier.setDrawings(this.drawings);
        sheetCopier.setImages(this.images);
        sheetCopier.setConditionalFormats(this.conditionalFormats);
        sheetCopier.setValidatedCells(this.validatedCells);
        sheetCopier.copySheet();
        this.dataValidation = sheetCopier.getDataValidation();
        this.comboBox = sheetCopier.getComboBox();
        this.plsRecord = sheetCopier.getPLSRecord();
        this.chartOnly = sheetCopier.isChartOnly();
        this.buttonPropertySet = sheetCopier.getButtonPropertySet();
        this.numRows = sheetCopier.getRows();
        this.autoFilter = sheetCopier.getAutoFilter();
        this.maxRowOutlineLevel = sheetCopier.getMaxRowOutlineLevel();
        this.maxColumnOutlineLevel = sheetCopier.getMaxColumnOutlineLevel();
    }

    void copy(WritableSheet writableSheet) {
        this.settings = new SheetSettings(writableSheet.getSettings(), this);
        WritableSheetImpl writableSheetImpl = (WritableSheetImpl)writableSheet;
        WritableSheetCopier writableSheetCopier = new WritableSheetCopier(writableSheet, this);
        writableSheetCopier.setColumnFormats(writableSheetImpl.columnFormats, this.columnFormats);
        writableSheetCopier.setMergedCells(writableSheetImpl.mergedCells, this.mergedCells);
        writableSheetCopier.setRows(writableSheetImpl.rows);
        writableSheetCopier.setRowBreaks(writableSheetImpl.rowBreaks, this.rowBreaks);
        writableSheetCopier.setColumnBreaks(writableSheetImpl.columnBreaks, this.columnBreaks);
        writableSheetCopier.setDataValidation(writableSheetImpl.dataValidation);
        writableSheetCopier.setSheetWriter(this.sheetWriter);
        writableSheetCopier.setDrawings(writableSheetImpl.drawings, this.drawings, this.images);
        writableSheetCopier.setWorkspaceOptions(writableSheetImpl.getWorkspaceOptions());
        writableSheetCopier.setPLSRecord(writableSheetImpl.plsRecord);
        writableSheetCopier.setButtonPropertySetRecord(writableSheetImpl.buttonPropertySet);
        writableSheetCopier.setHyperlinks(writableSheetImpl.hyperlinks, this.hyperlinks);
        writableSheetCopier.setValidatedCells(this.validatedCells);
        writableSheetCopier.copySheet();
        this.dataValidation = writableSheetCopier.getDataValidation();
        this.plsRecord = writableSheetCopier.getPLSRecord();
        this.buttonPropertySet = writableSheetCopier.getButtonPropertySet();
    }

    @Override
    public Cell findCell(String string) {
        return new CellFinder(this).findCell(string);
    }

    @Override
    public Cell findCell(String string, int n, int n2, int n3, int n4, boolean bl) {
        return new CellFinder(this).findCell(string, n, n2, n3, n4, bl);
    }

    @Override
    public Cell findCell(Pattern pattern, int n, int n2, int n3, int n4, boolean bl) {
        return new CellFinder(this).findCell(pattern, n, n2, n3, n4, bl);
    }

    @Override
    public LabelCell findLabelCell(String string) {
        return new CellFinder(this).findLabelCell(string);
    }

    @Override
    public Cell getCell(int n, int n2) {
        return this.getWritableCell(n, n2);
    }

    @Override
    public Cell getCell(String string) {
        return this.getCell(CellReferenceHelper.getColumn(string), CellReferenceHelper.getRow(string));
    }

    Chart[] getCharts() {
        return this.sheetWriter.getCharts();
    }

    @Override
    public Cell[] getColumn(int n) {
        int n2 = this.numRows - 1;
        int n3 = 0;
        boolean bl = false;
        while (n2 >= 0 && !bl) {
            if (this.getCell(n, n2).getType() != CellType.EMPTY) {
                bl = true;
                continue;
            }
            --n2;
        }
        Cell[] arrcell = new Cell[n2 + 1];
        while (n3 <= n2) {
            arrcell[n3] = this.getCell(n, n3);
            ++n3;
        }
        return arrcell;
    }

    @Override
    public CellFormat getColumnFormat(int n) {
        return this.getColumnView(n).getFormat();
    }

    ColumnInfoRecord getColumnInfo(int n) {
        Iterator iterator = this.columnFormats.iterator();
        boolean bl = false;
        ColumnInfoRecord columnInfoRecord = null;
        while (iterator.hasNext() && !bl) {
            columnInfoRecord = (ColumnInfoRecord)iterator.next();
            if (columnInfoRecord.getColumn() < n) continue;
            bl = true;
        }
        if (!bl) {
            return null;
        }
        int n2 = columnInfoRecord.getColumn();
        ColumnInfoRecord columnInfoRecord2 = null;
        if (n2 == n) {
            columnInfoRecord2 = columnInfoRecord;
        }
        return columnInfoRecord2;
    }

    @Override
    public int[] getColumnPageBreaks() {
        int[] arrn = new int[this.columnBreaks.size()];
        Iterator iterator = this.columnBreaks.iterator();
        int n = 0;
        while (iterator.hasNext()) {
            arrn[n] = (Integer)iterator.next();
            ++n;
        }
        return arrn;
    }

    @Override
    public CellView getColumnView(int n) {
        ColumnInfoRecord columnInfoRecord = this.getColumnInfo(n);
        CellView cellView = new CellView();
        if (columnInfoRecord != null) {
            cellView.setDimension(columnInfoRecord.getWidth() / 256);
            cellView.setSize(columnInfoRecord.getWidth());
            cellView.setHidden(columnInfoRecord.getHidden());
            cellView.setFormat(columnInfoRecord.getCellFormat());
            return cellView;
        }
        cellView.setDimension(this.settings.getDefaultColumnWidth() / 256);
        cellView.setSize(256 * this.settings.getDefaultColumnWidth());
        return cellView;
    }

    @Override
    public int getColumnWidth(int n) {
        return this.getColumnView(n).getDimension();
    }

    @Override
    public int getColumns() {
        return this.numColumns;
    }

    ComboBox getComboBox() {
        return this.comboBox;
    }

    public DataValidation getDataValidation() {
        return this.dataValidation;
    }

    @Override
    public Image getDrawing(int n) {
        return (Image)this.images.get(n);
    }

    final FooterRecord getFooter() {
        return this.sheetWriter.getFooter();
    }

    final HeaderRecord getHeader() {
        return this.sheetWriter.getHeader();
    }

    @Override
    public Hyperlink[] getHyperlinks() {
        Hyperlink[] arrhyperlink = new Hyperlink[this.hyperlinks.size()];
        for (int i = 0; i < this.hyperlinks.size(); ++i) {
            arrhyperlink[i] = (Hyperlink)this.hyperlinks.get(i);
        }
        return arrhyperlink;
    }

    @Override
    public WritableImage getImage(int n) {
        return (WritableImage)this.images.get(n);
    }

    @Override
    public Range[] getMergedCells() {
        return this.mergedCells.getMergedCells();
    }

    @Override
    public String getName() {
        return this.name;
    }

    @Override
    public int getNumberOfImages() {
        return this.images.size();
    }

    @Override
    public Cell[] getRow(int n) {
        int n2 = this.numColumns - 1;
        int n3 = 0;
        boolean bl = false;
        while (n2 >= 0 && !bl) {
            if (this.getCell(n2, n).getType() != CellType.EMPTY) {
                bl = true;
                continue;
            }
            --n2;
        }
        Cell[] arrcell = new Cell[n2 + 1];
        while (n3 <= n2) {
            arrcell[n3] = this.getCell(n3, n);
            ++n3;
        }
        return arrcell;
    }

    @Override
    public int getRowHeight(int n) {
        return this.getRowView(n).getDimension();
    }

    RowRecord getRowInfo(int n) {
        RowRecord[] arrrowRecord;
        if (n >= 0 && n <= (arrrowRecord = this.rows).length) {
            return arrrowRecord[n];
        }
        return null;
    }

    @Override
    public int[] getRowPageBreaks() {
        int[] arrn = new int[this.rowBreaks.size()];
        Iterator iterator = this.rowBreaks.iterator();
        int n = 0;
        while (iterator.hasNext()) {
            arrn[n] = (Integer)iterator.next();
            ++n;
        }
        return arrn;
    }

    RowRecord getRowRecord(int n) throws RowsExceededException {
        if (n < 65536) {
            RowRecord rowRecord;
            RowRecord[] arrrowRecord = this.rows;
            if (n >= arrrowRecord.length) {
                this.rows = new RowRecord[Math.max((int)(10 + arrrowRecord.length), (int)(n + 1))];
                System.arraycopy((Object)arrrowRecord, (int)0, (Object)this.rows, (int)0, (int)arrrowRecord.length);
            }
            if ((rowRecord = this.rows[n]) == null) {
                this.rows[n] = rowRecord = new RowRecord(n, this);
            }
            return rowRecord;
        }
        throw new RowsExceededException();
    }

    @Override
    public CellView getRowView(int n) {
        CellView cellView;
        block5 : {
            RowRecord rowRecord;
            block6 : {
                cellView = new CellView();
                try {
                    rowRecord = this.getRowRecord(n);
                    if (rowRecord == null) break block5;
                }
                catch (RowsExceededException rowsExceededException) {
                    cellView.setDimension(this.settings.getDefaultRowHeight());
                    cellView.setSize(this.settings.getDefaultRowHeight());
                    return cellView;
                }
                if (rowRecord.isDefaultHeight()) break block5;
                if (!rowRecord.isCollapsed()) break block6;
                cellView.setHidden(true);
                return cellView;
            }
            cellView.setDimension(rowRecord.getRowHeight());
            cellView.setSize(rowRecord.getRowHeight());
            return cellView;
        }
        cellView.setDimension(this.settings.getDefaultRowHeight());
        cellView.setSize(this.settings.getDefaultRowHeight());
        return cellView;
    }

    @Override
    public int getRows() {
        return this.numRows;
    }

    @Override
    public SheetSettings getSettings() {
        return this.settings;
    }

    WritableWorkbookImpl getWorkbook() {
        return this.workbook;
    }

    WorkbookSettings getWorkbookSettings() {
        return this.workbookSettings;
    }

    @Override
    public WritableCell getWritableCell(int n, int n2) {
        RowRecord[] arrrowRecord = this.rows;
        WritableCell writableCell = n2 < arrrowRecord.length && arrrowRecord[n2] != null ? arrrowRecord[n2].getCell(n) : null;
        if (writableCell == null) {
            writableCell = new EmptyCell(n, n2);
        }
        return writableCell;
    }

    @Override
    public WritableCell getWritableCell(String string) {
        return this.getWritableCell(CellReferenceHelper.getColumn(string), CellReferenceHelper.getRow(string));
    }

    @Override
    public WritableHyperlink[] getWritableHyperlinks() {
        WritableHyperlink[] arrwritableHyperlink = new WritableHyperlink[this.hyperlinks.size()];
        for (int i = 0; i < this.hyperlinks.size(); ++i) {
            arrwritableHyperlink[i] = (WritableHyperlink)this.hyperlinks.get(i);
        }
        return arrwritableHyperlink;
    }

    void importSheet(Sheet sheet) {
        this.settings = new SheetSettings(sheet.getSettings(), this);
        SheetCopier sheetCopier = new SheetCopier(sheet, this);
        sheetCopier.setColumnFormats(this.columnFormats);
        sheetCopier.setFormatRecords(this.formatRecords);
        sheetCopier.setHyperlinks(this.hyperlinks);
        sheetCopier.setMergedCells(this.mergedCells);
        sheetCopier.setRowBreaks(this.rowBreaks);
        sheetCopier.setColumnBreaks(this.columnBreaks);
        sheetCopier.setSheetWriter(this.sheetWriter);
        sheetCopier.setDrawings(this.drawings);
        sheetCopier.setImages(this.images);
        sheetCopier.setValidatedCells(this.validatedCells);
        sheetCopier.importSheet();
        this.dataValidation = sheetCopier.getDataValidation();
        this.comboBox = sheetCopier.getComboBox();
        this.plsRecord = sheetCopier.getPLSRecord();
        this.chartOnly = sheetCopier.isChartOnly();
        this.buttonPropertySet = sheetCopier.getButtonPropertySet();
        this.numRows = sheetCopier.getRows();
        this.maxRowOutlineLevel = sheetCopier.getMaxRowOutlineLevel();
        this.maxColumnOutlineLevel = sheetCopier.getMaxColumnOutlineLevel();
    }

    @Override
    public void insertColumn(int n) {
        if (n >= 0) {
            DataValidation dataValidation;
            ArrayList arrayList;
            if (n >= this.numColumns) {
                return;
            }
            for (int i = 0; i < this.numRows; ++i) {
                RowRecord[] arrrowRecord = this.rows;
                if (arrrowRecord[i] == null) continue;
                arrrowRecord[i].insertColumn(n);
            }
            Iterator iterator = this.hyperlinks.iterator();
            while (iterator.hasNext()) {
                ((HyperlinkRecord)iterator.next()).insertColumn(n);
            }
            for (ColumnInfoRecord columnInfoRecord : this.columnFormats) {
                if (columnInfoRecord.getColumn() < n) continue;
                columnInfoRecord.incrementColumn();
            }
            if (this.autosizedColumns.size() > 0) {
                TreeSet treeSet = new TreeSet();
                for (Integer n2 : this.autosizedColumns) {
                    if (n2 >= n) {
                        treeSet.add((Object)new Integer(1 + n2));
                        continue;
                    }
                    treeSet.add((Object)n2);
                }
                this.autosizedColumns = treeSet;
            }
            if ((dataValidation = this.dataValidation) != null) {
                dataValidation.insertColumn(n);
            }
            if ((arrayList = this.validatedCells) != null && arrayList.size() > 0) {
                Iterator iterator2 = this.validatedCells.iterator();
                while (iterator2.hasNext()) {
                    CellFeatures cellFeatures = ((CellValue)iterator2.next()).getCellFeatures();
                    if (cellFeatures.getDVParser() == null) continue;
                    cellFeatures.getDVParser().insertColumn(n);
                }
            }
            this.mergedCells.insertColumn(n);
            ArrayList arrayList2 = new ArrayList();
            Iterator iterator3 = this.columnBreaks.iterator();
            while (iterator3.hasNext()) {
                int n3 = (Integer)iterator3.next();
                if (n3 >= n) {
                    ++n3;
                }
                arrayList2.add((Object)new Integer(n3));
            }
            this.columnBreaks = arrayList2;
            Iterator iterator4 = this.conditionalFormats.iterator();
            while (iterator4.hasNext()) {
                ((ConditionalFormat)iterator4.next()).insertColumn(n);
            }
            if (this.workbookSettings.getFormulaAdjust()) {
                this.workbook.columnInserted(this, n);
            }
            this.numColumns = 1 + this.numColumns;
        }
    }

    @Override
    public void insertRow(int n) {
        if (n >= 0) {
            ArrayList arrayList;
            int n2;
            int n3 = this.numRows;
            if (n >= n3) {
                return;
            }
            RowRecord[] arrrowRecord = this.rows;
            this.rows = n3 == arrrowRecord.length ? new RowRecord[10 + arrrowRecord.length] : new RowRecord[arrrowRecord.length];
            System.arraycopy((Object)arrrowRecord, (int)0, (Object)this.rows, (int)0, (int)n);
            RowRecord[] arrrowRecord2 = this.rows;
            System.arraycopy((Object)arrrowRecord, (int)n, (Object)arrrowRecord2, (int)n2, (int)(this.numRows - n));
            for (n2 = n + 1; n2 <= this.numRows; ++n2) {
                RowRecord[] arrrowRecord3 = this.rows;
                if (arrrowRecord3[n2] == null) continue;
                arrrowRecord3[n2].incrementRow();
            }
            Iterator iterator = this.hyperlinks.iterator();
            while (iterator.hasNext()) {
                ((HyperlinkRecord)iterator.next()).insertRow(n);
            }
            DataValidation dataValidation = this.dataValidation;
            if (dataValidation != null) {
                dataValidation.insertRow(n);
            }
            if ((arrayList = this.validatedCells) != null && arrayList.size() > 0) {
                Iterator iterator2 = this.validatedCells.iterator();
                while (iterator2.hasNext()) {
                    CellFeatures cellFeatures = ((CellValue)iterator2.next()).getCellFeatures();
                    if (cellFeatures.getDVParser() == null) continue;
                    cellFeatures.getDVParser().insertRow(n);
                }
            }
            this.mergedCells.insertRow(n);
            ArrayList arrayList2 = new ArrayList();
            Iterator iterator3 = this.rowBreaks.iterator();
            while (iterator3.hasNext()) {
                int n4 = (Integer)iterator3.next();
                if (n4 >= n) {
                    ++n4;
                }
                arrayList2.add((Object)new Integer(n4));
            }
            this.rowBreaks = arrayList2;
            Iterator iterator4 = this.conditionalFormats.iterator();
            while (iterator4.hasNext()) {
                ((ConditionalFormat)iterator4.next()).insertRow(n);
            }
            if (this.workbookSettings.getFormulaAdjust()) {
                this.workbook.rowInserted(this, n);
            }
            this.numRows = 1 + this.numRows;
        }
    }

    boolean isChartOnly() {
        return this.chartOnly;
    }

    @Override
    public boolean isHidden() {
        return this.settings.isHidden();
    }

    @Override
    public boolean isProtected() {
        return this.settings.isProtected();
    }

    @Override
    public Range mergeCells(int n, int n2, int n3, int n4) throws WriteException, RowsExceededException {
        if (n3 < n || n4 < n2) {
            logger.warn("Cannot merge cells - top left and bottom right incorrectly specified");
        }
        if (n3 >= this.numColumns || n4 >= this.numRows) {
            this.addCell(new Blank(n3, n4));
        }
        SheetRangeImpl sheetRangeImpl = new SheetRangeImpl(this, n, n2, n3, n4);
        this.mergedCells.add(sheetRangeImpl);
        return sheetRangeImpl;
    }

    void rationalize(IndexMapping indexMapping, IndexMapping indexMapping2, IndexMapping indexMapping3) {
        RowRecord[] arrrowRecord;
        Iterator iterator = this.columnFormats.iterator();
        while (iterator.hasNext()) {
            ((ColumnInfoRecord)iterator.next()).rationalize(indexMapping);
        }
        int n = 0;
        for (int i = 0; i < (arrrowRecord = this.rows).length; ++i) {
            if (arrrowRecord[i] == null) continue;
            arrrowRecord[i].rationalize(indexMapping);
        }
        Chart[] arrchart = this.getCharts();
        while (n < arrchart.length) {
            arrchart[n].rationalize(indexMapping, indexMapping2, indexMapping3);
            ++n;
        }
    }

    @Override
    public void removeColumn(int n) {
        if (n >= 0) {
            ArrayList arrayList;
            if (n >= this.numColumns) {
                return;
            }
            for (int i = 0; i < this.numRows; ++i) {
                RowRecord[] arrrowRecord = this.rows;
                if (arrrowRecord[i] == null) continue;
                arrrowRecord[i].removeColumn(n);
            }
            Iterator iterator = this.hyperlinks.iterator();
            while (iterator.hasNext()) {
                HyperlinkRecord hyperlinkRecord = (HyperlinkRecord)iterator.next();
                if (hyperlinkRecord.getColumn() == n && hyperlinkRecord.getLastColumn() == n) {
                    iterator.remove();
                    continue;
                }
                hyperlinkRecord.removeColumn(n);
            }
            DataValidation dataValidation = this.dataValidation;
            if (dataValidation != null) {
                dataValidation.removeColumn(n);
            }
            if ((arrayList = this.validatedCells) != null && arrayList.size() > 0) {
                Iterator iterator2 = this.validatedCells.iterator();
                while (iterator2.hasNext()) {
                    CellFeatures cellFeatures = ((CellValue)iterator2.next()).getCellFeatures();
                    if (cellFeatures.getDVParser() == null) continue;
                    cellFeatures.getDVParser().removeColumn(n);
                }
            }
            this.mergedCells.removeColumn(n);
            ArrayList arrayList2 = new ArrayList();
            Iterator iterator3 = this.columnBreaks.iterator();
            while (iterator3.hasNext()) {
                int n2 = (Integer)iterator3.next();
                if (n2 == n) continue;
                if (n2 > n) {
                    --n2;
                }
                arrayList2.add((Object)new Integer(n2));
            }
            this.columnBreaks = arrayList2;
            Iterator iterator4 = this.columnFormats.iterator();
            ColumnInfoRecord columnInfoRecord = null;
            while (iterator4.hasNext()) {
                ColumnInfoRecord columnInfoRecord2 = (ColumnInfoRecord)iterator4.next();
                if (columnInfoRecord2.getColumn() == n) {
                    columnInfoRecord = columnInfoRecord2;
                    continue;
                }
                if (columnInfoRecord2.getColumn() <= n) continue;
                columnInfoRecord2.decrementColumn();
            }
            if (columnInfoRecord != null) {
                this.columnFormats.remove(columnInfoRecord);
            }
            if (this.autosizedColumns.size() > 0) {
                TreeSet treeSet = new TreeSet();
                for (Integer n3 : this.autosizedColumns) {
                    if (n3 == n) continue;
                    if (n3 > n) {
                        treeSet.add((Object)new Integer(-1 + n3));
                        continue;
                    }
                    treeSet.add((Object)n3);
                }
                this.autosizedColumns = treeSet;
            }
            Iterator iterator5 = this.conditionalFormats.iterator();
            while (iterator5.hasNext()) {
                ((ConditionalFormat)iterator5.next()).removeColumn(n);
            }
            if (this.workbookSettings.getFormulaAdjust()) {
                this.workbook.columnRemoved(this, n);
            }
            this.numColumns = -1 + this.numColumns;
        }
    }

    void removeDataValidation(CellValue cellValue) {
        ArrayList arrayList;
        DataValidation dataValidation = this.dataValidation;
        if (dataValidation != null) {
            dataValidation.removeDataValidation(cellValue.getColumn(), cellValue.getRow());
        }
        if ((arrayList = this.validatedCells) != null && !arrayList.remove((Object)cellValue)) {
            Logger logger = WritableSheetImpl.logger;
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("Could not remove validated cell ");
            stringBuilder.append(CellReferenceHelper.getCellReference(cellValue));
            logger.warn(stringBuilder.toString());
        }
    }

    void removeDrawing(DrawingGroupObject drawingGroupObject) {
        boolean bl;
        int n = this.drawings.size();
        this.drawings.remove((Object)drawingGroupObject);
        int n2 = this.drawings.size();
        this.drawingsModified = bl = true;
        if (n2 != n - bl) {
            bl = false;
        }
        Assert.verify(bl);
    }

    @Override
    public void removeHyperlink(WritableHyperlink writableHyperlink) {
        this.removeHyperlink(writableHyperlink, false);
    }

    @Override
    public void removeHyperlink(WritableHyperlink writableHyperlink, boolean bl) {
        ArrayList arrayList = this.hyperlinks;
        arrayList.remove(arrayList.indexOf((Object)writableHyperlink));
        if (!bl) {
            boolean bl2 = this.rows.length > writableHyperlink.getRow() && this.rows[writableHyperlink.getRow()] != null;
            Assert.verify(bl2);
            this.rows[writableHyperlink.getRow()].removeCell(writableHyperlink.getColumn());
        }
    }

    @Override
    public void removeImage(WritableImage writableImage) {
        this.drawings.remove((Object)writableImage);
        this.images.remove((Object)writableImage);
        this.drawingsModified = true;
        this.workbook.removeDrawing(writableImage);
    }

    @Override
    public void removeRow(int n) {
        if (n >= 0 && n < this.numRows) {
            ArrayList arrayList;
            RowRecord[] arrrowRecord = this.rows;
            this.rows = new RowRecord[arrrowRecord.length];
            System.arraycopy((Object)arrrowRecord, (int)0, (Object)this.rows, (int)0, (int)n);
            int n2 = n + 1;
            System.arraycopy((Object)arrrowRecord, (int)n2, (Object)this.rows, (int)n, (int)(this.numRows - n2));
            for (int i = n; i < this.numRows; ++i) {
                RowRecord[] arrrowRecord2 = this.rows;
                if (arrrowRecord2[i] == null) continue;
                arrrowRecord2[i].decrementRow();
            }
            Iterator iterator = this.hyperlinks.iterator();
            while (iterator.hasNext()) {
                HyperlinkRecord hyperlinkRecord = (HyperlinkRecord)iterator.next();
                if (hyperlinkRecord.getRow() == n && hyperlinkRecord.getLastRow() == n) {
                    iterator.remove();
                    continue;
                }
                hyperlinkRecord.removeRow(n);
            }
            DataValidation dataValidation = this.dataValidation;
            if (dataValidation != null) {
                dataValidation.removeRow(n);
            }
            if ((arrayList = this.validatedCells) != null && arrayList.size() > 0) {
                Iterator iterator2 = this.validatedCells.iterator();
                while (iterator2.hasNext()) {
                    CellFeatures cellFeatures = ((CellValue)iterator2.next()).getCellFeatures();
                    if (cellFeatures.getDVParser() == null) continue;
                    cellFeatures.getDVParser().removeRow(n);
                }
            }
            this.mergedCells.removeRow(n);
            ArrayList arrayList2 = new ArrayList();
            Iterator iterator3 = this.rowBreaks.iterator();
            while (iterator3.hasNext()) {
                int n3 = (Integer)iterator3.next();
                if (n3 == n) continue;
                if (n3 > n) {
                    --n3;
                }
                arrayList2.add((Object)new Integer(n3));
            }
            this.rowBreaks = arrayList2;
            Iterator iterator4 = this.conditionalFormats.iterator();
            while (iterator4.hasNext()) {
                ((ConditionalFormat)iterator4.next()).removeRow(n);
            }
            if (this.workbookSettings.getFormulaAdjust()) {
                this.workbook.rowRemoved(this, n);
            }
            this.numRows = -1 + this.numRows;
            return;
        }
        if (this.workbookSettings.getFormulaAdjust()) {
            this.workbook.rowRemoved(this, n);
        }
    }

    @Override
    public void removeSharedDataValidation(WritableCell writableCell) throws WriteException {
        WritableCellFeatures writableCellFeatures = writableCell.getWritableCellFeatures();
        if (writableCellFeatures != null) {
            if (!writableCellFeatures.hasDataValidation()) {
                return;
            }
            DVParser dVParser = writableCellFeatures.getDVParser();
            if (!dVParser.extendedCellsValidation()) {
                writableCellFeatures.removeDataValidation();
                return;
            }
            if (dVParser.extendedCellsValidation() && (writableCell.getColumn() != dVParser.getFirstColumn() || writableCell.getRow() != dVParser.getFirstRow())) {
                Logger logger = WritableSheetImpl.logger;
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.append("Cannot remove data validation from ");
                stringBuilder.append(CellReferenceHelper.getCellReference(dVParser.getFirstColumn(), dVParser.getFirstRow()));
                stringBuilder.append("-");
                stringBuilder.append(CellReferenceHelper.getCellReference(dVParser.getLastColumn(), dVParser.getLastRow()));
                stringBuilder.append(" because the selected cell ");
                stringBuilder.append(CellReferenceHelper.getCellReference(writableCell));
                stringBuilder.append(" is not the top left cell in the range");
                logger.warn(stringBuilder.toString());
                return;
            }
            for (int i = dVParser.getFirstRow(); i <= dVParser.getLastRow(); ++i) {
                for (int j = dVParser.getFirstColumn(); j <= dVParser.getLastColumn(); ++j) {
                    CellValue cellValue = this.rows[i].getCell(j);
                    if (cellValue == null) continue;
                    cellValue.getWritableCellFeatures().removeSharedDataValidation();
                    cellValue.removeCellFeatures();
                }
            }
            DataValidation dataValidation = this.dataValidation;
            if (dataValidation != null) {
                dataValidation.removeSharedDataValidation(dVParser.getFirstColumn(), dVParser.getFirstRow(), dVParser.getLastColumn(), dVParser.getLastRow());
            }
        }
    }

    @Override
    public void setColumnGroup(int n, int n2, boolean bl) throws WriteException, RowsExceededException {
        if (n2 < n) {
            logger.warn("Cannot merge cells - top and bottom rows incorrectly specified");
        }
        while (n <= n2) {
            ColumnInfoRecord columnInfoRecord = this.getColumnInfo(n);
            if (columnInfoRecord == null) {
                this.setColumnView(n, new CellView());
                columnInfoRecord = this.getColumnInfo(n);
            }
            columnInfoRecord.incrementOutlineLevel();
            columnInfoRecord.setCollapsed(bl);
            this.maxColumnOutlineLevel = Math.max((int)this.maxColumnOutlineLevel, (int)columnInfoRecord.getOutlineLevel());
            ++n;
        }
    }

    @Override
    public void setColumnView(int n, int n2) {
        CellView cellView = new CellView();
        cellView.setSize(n2 * 256);
        this.setColumnView(n, cellView);
    }

    @Override
    public void setColumnView(int n, int n2, CellFormat cellFormat) {
        CellView cellView = new CellView();
        cellView.setSize(n2 * 256);
        cellView.setFormat(cellFormat);
        this.setColumnView(n, cellView);
    }

    @Override
    public void setColumnView(int n, CellView cellView) {
        XFRecord xFRecord = (XFRecord)cellView.getFormat();
        if (xFRecord == null) {
            xFRecord = this.getWorkbook().getStyles().getNormalStyle();
        }
        try {
            if (!xFRecord.isInitialized()) {
                this.formatRecords.addStyle(xFRecord);
            }
            int n2 = cellView.depUsed() ? 256 * cellView.getDimension() : cellView.getSize();
            if (cellView.isAutosize()) {
                this.autosizedColumns.add((Object)new Integer(n));
            }
            ColumnInfoRecord columnInfoRecord = new ColumnInfoRecord(n, n2, xFRecord);
            if (cellView.isHidden()) {
                columnInfoRecord.setHidden(true);
            }
            if (!this.columnFormats.contains((Object)columnInfoRecord)) {
                this.columnFormats.add((Object)columnInfoRecord);
                return;
            }
            this.columnFormats.remove((Object)columnInfoRecord);
            this.columnFormats.add((Object)columnInfoRecord);
            return;
        }
        catch (NumFormatRecordsException numFormatRecordsException) {
            logger.warn("Maximum number of format records exceeded.  Using default format.");
            ColumnInfoRecord columnInfoRecord = new ColumnInfoRecord(n, 256 * cellView.getDimension(), WritableWorkbook.NORMAL_STYLE);
            if (!this.columnFormats.contains((Object)columnInfoRecord)) {
                this.columnFormats.add((Object)columnInfoRecord);
            }
            return;
        }
    }

    void setComboBox(ComboBox comboBox) {
        this.comboBox = comboBox;
    }

    @Override
    public void setFooter(String string, String string2, String string3) {
        HeaderFooter headerFooter = new HeaderFooter();
        headerFooter.getLeft().append(string);
        headerFooter.getCentre().append(string2);
        headerFooter.getRight().append(string3);
        this.settings.setFooter(headerFooter);
    }

    @Override
    public void setHeader(String string, String string2, String string3) {
        HeaderFooter headerFooter = new HeaderFooter();
        headerFooter.getLeft().append(string);
        headerFooter.getCentre().append(string2);
        headerFooter.getRight().append(string3);
        this.settings.setHeader(headerFooter);
    }

    @Override
    public void setHidden(boolean bl) {
        this.settings.setHidden(bl);
    }

    @Override
    public void setName(String string) {
        this.name = string;
    }

    @Override
    public void setPageSetup(PageOrientation pageOrientation) {
        this.settings.setOrientation(pageOrientation);
    }

    @Override
    public void setPageSetup(PageOrientation pageOrientation, double d, double d2) {
        this.settings.setOrientation(pageOrientation);
        this.settings.setHeaderMargin(d);
        this.settings.setFooterMargin(d2);
    }

    @Override
    public void setPageSetup(PageOrientation pageOrientation, PaperSize paperSize, double d, double d2) {
        this.settings.setPaperSize(paperSize);
        this.settings.setOrientation(pageOrientation);
        this.settings.setHeaderMargin(d);
        this.settings.setFooterMargin(d2);
    }

    @Override
    public void setProtected(boolean bl) {
        this.settings.setProtected(bl);
    }

    @Override
    public void setRowGroup(int n, int n2, boolean bl) throws WriteException, RowsExceededException {
        if (n2 < n) {
            logger.warn("Cannot merge cells - top and bottom rows incorrectly specified");
        }
        while (n <= n2) {
            RowRecord rowRecord = this.getRowRecord(n);
            this.numRows = Math.max((int)(++n), (int)this.numRows);
            rowRecord.incrementOutlineLevel();
            rowRecord.setCollapsed(bl);
            this.maxRowOutlineLevel = Math.max((int)this.maxRowOutlineLevel, (int)rowRecord.getOutlineLevel());
        }
    }

    @Override
    public void setRowView(int n, int n2) throws RowsExceededException {
        CellView cellView = new CellView();
        cellView.setSize(n2);
        cellView.setHidden(false);
        this.setRowView(n, cellView);
    }

    @Override
    public void setRowView(int n, int n2, boolean bl) throws RowsExceededException {
        CellView cellView = new CellView();
        cellView.setSize(n2);
        cellView.setHidden(bl);
        this.setRowView(n, cellView);
    }

    /*
     * Exception decompiling
     */
    @Override
    public void setRowView(int var1, CellView var2) throws RowsExceededException {
        // This method has failed to decompile.  When submitting a bug report, please provide this stack trace, and (if you hold appropriate legal rights) the relevant class file.
        // org.benf.cfr.reader.util.ConfusedCFRException: Invalid stack depths @ lbl24 : ALOAD : trying to set 1 previously set to 0
        // org.benf.cfr.reader.b.a.a.g.a(Op02WithProcessedDataAndRefs.java:203)
        // org.benf.cfr.reader.b.a.a.g.a(Op02WithProcessedDataAndRefs.java:1489)
        // org.benf.cfr.reader.b.f.a(CodeAnalyser.java:308)
        // org.benf.cfr.reader.b.f.a(CodeAnalyser.java:182)
        // org.benf.cfr.reader.b.f.a(CodeAnalyser.java:127)
        // org.benf.cfr.reader.entities.attributes.f.c(AttributeCode.java:96)
        // org.benf.cfr.reader.entities.g.p(Method.java:396)
        // org.benf.cfr.reader.entities.d.e(ClassFile.java:890)
        // org.benf.cfr.reader.entities.d.b(ClassFile.java:792)
        // org.benf.cfr.reader.b.a(Driver.java:128)
        // org.benf.cfr.reader.a.a(CfrDriverImpl.java:63)
        // com.njlabs.showjava.decompilers.JavaExtractionWorker.decompileWithCFR(JavaExtractionWorker.kt:61)
        // com.njlabs.showjava.decompilers.JavaExtractionWorker.doWork(JavaExtractionWorker.kt:130)
        // com.njlabs.showjava.decompilers.BaseDecompiler.withAttempt(BaseDecompiler.kt:108)
        // com.njlabs.showjava.workers.DecompilerWorker$b.run(DecompilerWorker.kt:118)
        // java.util.concurrent.ThreadPoolExecutor.runWorker(ThreadPoolExecutor.java:1167)
        // java.util.concurrent.ThreadPoolExecutor$Worker.run(ThreadPoolExecutor.java:641)
        // java.lang.Thread.run(Thread.java:919)
        throw new IllegalStateException("Decompilation failed");
    }

    @Override
    public void setRowView(int n, boolean bl) throws RowsExceededException {
        CellView cellView = new CellView();
        cellView.setHidden(bl);
        this.setRowView(n, cellView);
    }

    public void setSelected() {
        this.settings.setSelected();
    }

    @Override
    public void unmergeCells(Range range) {
        this.mergedCells.unmergeCells(range);
    }

    @Override
    public void unsetColumnGroup(int n, int n2) throws WriteException, RowsExceededException {
        if (n2 < n) {
            logger.warn("Cannot merge cells - top and bottom rows incorrectly specified");
        }
        while (n <= n2) {
            this.getColumnInfo(n).decrementOutlineLevel();
            ++n;
        }
        this.maxColumnOutlineLevel = 0;
        for (ColumnInfoRecord columnInfoRecord : this.columnFormats) {
            this.maxColumnOutlineLevel = Math.max((int)this.maxColumnOutlineLevel, (int)columnInfoRecord.getOutlineLevel());
        }
    }

    @Override
    public void unsetRowGroup(int n, int n2) throws WriteException, RowsExceededException {
        if (n2 < n) {
            logger.warn("Cannot merge cells - top and bottom rows incorrectly specified");
        }
        if (n2 >= this.numRows) {
            Logger logger = WritableSheetImpl.logger;
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("");
            stringBuilder.append(n2);
            stringBuilder.append(" is greater than the sheet bounds");
            logger.warn(stringBuilder.toString());
            n2 = -1 + this.numRows;
        }
        while (n <= n2) {
            this.rows[n].decrementOutlineLevel();
            ++n;
        }
        this.maxRowOutlineLevel = 0;
        int n3 = this.rows.length;
        do {
            int n4 = n3 - 1;
            if (n3 <= 0) break;
            this.maxRowOutlineLevel = Math.max((int)this.maxRowOutlineLevel, (int)this.rows[n4].getOutlineLevel());
            n3 = n4;
        } while (true);
    }

    public void write() throws IOException {
        boolean bl = this.drawingsModified;
        if (this.workbook.getDrawingGroup() != null) {
            bl |= this.workbook.getDrawingGroup().hasDrawingsOmitted();
        }
        if (this.autosizedColumns.size() > 0) {
            this.autosizeColumns();
        }
        this.sheetWriter.setWriteData(this.rows, this.rowBreaks, this.columnBreaks, this.hyperlinks, this.mergedCells, this.columnFormats, this.maxRowOutlineLevel, this.maxColumnOutlineLevel);
        this.sheetWriter.setDimensions(this.getRows(), this.getColumns());
        this.sheetWriter.setSettings(this.settings);
        this.sheetWriter.setPLS(this.plsRecord);
        this.sheetWriter.setDrawings(this.drawings, bl);
        this.sheetWriter.setButtonPropertySet(this.buttonPropertySet);
        this.sheetWriter.setDataValidation(this.dataValidation, this.validatedCells);
        this.sheetWriter.setConditionalFormats(this.conditionalFormats);
        this.sheetWriter.setAutoFilter(this.autoFilter);
        this.sheetWriter.write();
    }

}

